package ssw.annotation;

import java.util.List;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.Viewer;
import ssw.annotation.AnnotationFileHandler.AnnoObject;

/*
 * Created on 14.02.2005
 */
/**
 *   Copyright (C) 2005  Stefan Loidl
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *   
 *   eMail: stefanloidl@gmx.net
 *   
 * This class provides the content for both, the annotation list in the editor and the view. 
 * The background data of this provider is a list (java.util) which contains AnnoObjects. 
 * This list for example can be easily extracted form an AnnotationFile object. The objects are 
 * directly distributed without any modification. The input change tracking possibility 
 * is not used yet.
 *  
 * @version 1.0.0
 * @author Loidl Stefan, 0255886 
 */

public class AnnoContentProvider implements IStructuredContentProvider {

	private List annoList=null;
	
	/**
	 * Meant to pass changes- not used
	 */
	public void inputChanged(Viewer v, Object oldInput, Object newInput) {}
	
	/**
	 * Not used- no hard resources allocated
	 */
	public void dispose() {}
	
	/**
	 * Externally set the annotation list
	 * @param l external List
	 */
	public void setAnnotationList(List l){
		annoList=l;
	}
	
	/**
	 * Add an Annotation Object to the list 
	 * @param a Item to add
	 * @return Success?
	 */
	public boolean addToAnnotationList(AnnoObject a){
		if(annoList==null) return false;
		return annoList.add(a);
	}
	
	/**
	 * Retrive the list elements. 
	 */
	public Object[] getElements(Object parent) {
		if(annoList==null || annoList.isEmpty()) return new Object[0];
		return annoList.toArray();
	}
}
