package ssw.annotation.AnnotationFileHandler;
import java.util.Hashtable;
import java.util.List;

/*
 * Created on 08.03.2005
 */

/**
 *   Copyright (C) 2005  Stefan Loidl
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *   
 *   eMail: stefanloidl@gmx.net
 *   
 * This interface describes the structure of an annotation file handler. 
 * As annotations are not specific to any file type it could be necessary to 
 * adopt the file handler to match another goal. If for example annotations 
 * should be implemented for a completely new file type, which includes annotations 
 * and other data this interface has to be extended.
 * It defines save and load functionality, and how to handle annotations and global 
 * data. 
 * Global data is defined as key- value string pairs, while annotations are handled 
 * via the interface AnnotationItem. 
 * 
 * @version 1.0.0
 * @author Loidl Stefan, 0255886
 */
public interface AnnotationFileHandler {
	
	/**
	 * Reads an annotation file with a defined structure.
	 * @param file- relative file path (Workspace)
	 * @return success?
	 */
	public boolean readFile(String file);
	
	/**
	 * Saves the abstact file to a physical one.
	 * @return success?
	 */
	public boolean saveFile();
	
	/**
	 * Saves the abstact file to a physical one.
	 * @param file- relative file path (Workspace)
	 * @return success?
	 */
	public boolean saveFile(String file);
	
	/**
	 * Adds a new Element to the file header.
	 * @param key
	 * @param value
	 */
	public void addHeaderItem(String key, String value);
	
	/**
	 * Adds a new annotation item to the file
	 * @param anno
	 */
	public void addAnnotationItem(AnnotationItem anno);
	
	/**
	 * Returns a list of the annotations.
	 * @return
	 */
	public List getAnnotationItems();
	
	/**
	 * Returns a hashtable with the header elements.
	 * @return
	 */
	public Hashtable getHeaderItems();
	
	/**
	 * Returns the filename.
	 * @return
	 */
	public String getFileName();
	
	/**
	 * Refreshes the data from the base file.
	 * @return
	 */
	public boolean refresh();

}
