package ssw.annotation.annoedit;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.IResourceDelta;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowData;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.*;
import org.eclipse.ui.part.MultiPageEditorPart;
import ssw.annotation.AnnoContentProvider;
import ssw.annotation.AnnotationPlugin;
import ssw.annotation.AnnotationFileHandler.AnnoObject;
import ssw.annotation.AnnotationFileHandler.AnnotationFile;
import ssw.annotation.Preferences.AnnoPreferencePage;
import ssw.annotation.annoview.AnnotationViewUtils;
import ssw.annotation.annoview.dialogs.AnnoInputDialog;

/*
 * Created on 14.02.2005
 */

/**
 *   Copyright (C) 2005  Stefan Loidl
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *   
 *   eMail: stefanloidl@gmx.net
 *   
 * The Annotation Editor is a multi tab editor providing the possibility to get a brief 
 * overview over the annotations (first tab) - this is meant to be the information the 
 * student is interested in when receiving the annotated work. This tab moreover provides 
 * the possibility to change the general information of the annotation file. 
 * The second tab contains the information about the annotations. It is possible to change 
 * annotations using a double click action which opens an AnnotationInputDialog. Furthermore 
 * the user has the possibility to remove items from the list. 
 * The editor is configured (plugin.xml) to work with *.ann files. These files are strictly 
 * following the definition of the plug-ins annotation file structure. The editor loads the 
 * whole file content into an AnnotationFile and its content provider. The file resources 
 * have to be in synch with the internal Eclipse workspace resources. If the resource changes, 
 * e.g. through inserting a new annotation, the editor actualizes the whole annotation 
 * information (reloads the file). The editor reacts on a resource change event of the current 
 * project. For example if the project is closed, the editor is also closed.
 * The save infrastructure of Eclipse is not used within this editor. If an annotation is 
 * changed the changes are synchronized with the file immediately. Therefore the save as 
 * option is not available too. Annotation files generated by the system use a default annotation 
 * name. The system itself doesnt provide the possibility to change the filename. Nevertheless 
 * the editor is able to open all *.ann files.  
 * 
 * <ul>
 * <li>page 0 :brief overview- author, points  
 * <li>page 1 :Annotation list/editing
 * </ul>
 * 
 * @version 1.0.0
 * @author Loidl Stefan, 0255886
 */

public class AnnotationEditor extends MultiPageEditorPart implements IResourceChangeListener{

	/*Page indizes of the two editor pages*/
	private int globEditorPageIndex;
	private int editorPageIndex;

	/*Textfields in Global Editor Tab*/
	private Text nameTextField;
	private Text eMailTextField;
	private Text maxPointsTextField;
	
	/*Color definitions*/
	private Color colWhite=new Color(null,255,255,255);
	
	/*Labels used in Global Editor Tab (Annotation Information)*/
	private Label annoCountLabel;
	private Label totalScore;
	private Label totalModification;

	/**
	 * 
	 * @uml.property name="contentProvider"
	 * @uml.associationEnd multiplicity="(0 1)"
	 */
	/*Editorpage Elements*/
	private AnnoContentProvider contentProvider = new AnnoContentProvider();

	/**
	 * 
	 * @uml.property name="annoFile"
	 * @uml.associationEnd multiplicity="(0 1)"
	 */
	private AnnotationFile annoFile = new AnnotationFile();

	//File to be shown could be loaded
	private boolean fileOK=false;
	private Table table;
	private TableViewer viewer;
	
	/*Actions*/
	private Action doubleClickAction;
	private Action actionRemove;
	
	/*Tabel headings*/
	private String[] columnLabels=new String[3];
	
	/*Resource file*/
	private IFile file;
	
	/*Editorinput used on resourceChange event*/
	private IEditorInput editorInput;
	
	/*Editor Context Menu ID/Type*/
	private final String CMenuID="#AnnoEditPopup";
	private final String CMenuType="#PopupMenu";
	
	/**
	 * Constructor
	 */
	public AnnotationEditor() {
		super();
		columnLabels[0]=AnnotationPlugin.getResourceString("AE_C_DESCRIP");
		columnLabels[1]=AnnotationPlugin.getResourceString("AE_C_POINTS");
		columnLabels[2]=AnnotationPlugin.getResourceString("AE_C_FILE");
		ResourcesPlugin.getWorkspace().addResourceChangeListener(this);
	}

	/**
	 * Create multipage tabs.
	 */
	protected void createPages() {
		createGlobalEditorPage();
		createTableEditorPage();
	}
	
	/**
	 * Creates the Table Editor tab which contains the list of
	 * the annotations.
	 */
	void createTableEditorPage() {
		
		Composite parent = new Composite(getContainer(), SWT.NONE);
		GridLayout layout = new GridLayout();
		parent.setLayout(layout);
		layout.numColumns = 2;

		table=new Table(parent,SWT.SINGLE | SWT.H_SCROLL | SWT.V_SCROLL|
				SWT.BORDER|SWT.FULL_SELECTION);
			
		//******define Gridlayout******
		GridData grid=new GridData(GridData.FILL_BOTH);
		grid.grabExcessVerticalSpace=true;
		grid.horizontalSpan=3;
		table.setLayoutData(grid);
		table.setLinesVisible(true);
		table.setHeaderVisible(true);
		
		//********Create Colums*******	
		createColumns(table);
		
		//********Create Viewer*******
		viewer= new TableViewer(table);
		viewer.setContentProvider(contentProvider);
		viewer.setLabelProvider(new AnnoEditLabelProvider());
		viewer.setInput(getEditorSite());
		
		editorPageIndex = addPage(parent);
		setPageText(editorPageIndex, 
				AnnotationPlugin.getResourceString("AE_ET_EDITTAB"));
		
		hookDoubleClickAction();
		hookContextMenu();
	}
	
	/**
	 * Create the columns for the editor tabel. 
	 */
	private void createColumns(Table table) {
		TableColumn column;
		
		column=new TableColumn(table,SWT.LEFT,0);
		column.setText(columnLabels[0]);
		column.setWidth(500);
		column.setResizable(true);
		
		column=new TableColumn(table,SWT.CENTER,1);
		column.setText(columnLabels[1]);
		column.setWidth(100);
		column.setResizable(true);

		column=new TableColumn(table,SWT.LEFT,2);
		column.setText(columnLabels[2]);
		column.setWidth(150);
		column.setResizable(true);
	}
	
	/**
	 * Creates the Global Information Page of the editor containing
	 * possibilities to change heather information.
	 */
	void createGlobalEditorPage() {
		Composite composite = new Composite(getContainer(), SWT.NONE);
		composite.setBackground(colWhite);
		RowLayout editLayout=new RowLayout(SWT.VERTICAL);
		editLayout.marginLeft=20;
		editLayout.marginTop=20;
		composite.setLayout(editLayout);
		
		//*********define Layout**********
	    FillLayout authorLayout=new FillLayout(SWT.VERTICAL);
	    authorLayout.marginHeight=10;
	    authorLayout.marginWidth=10;
	    authorLayout.spacing=3;
		
	    //*********General Information Group*******
	    Group authorInfo=new Group(composite,SWT.FILL);
	    authorInfo.setText(AnnotationPlugin.getResourceString("AE_GL_GRGENERAL"));
	    authorInfo.setLayout(authorLayout);
	    authorInfo.setLayoutData(new RowData(350,165));
	    authorInfo.setBackground(colWhite);
		
		Label nameLabel=new Label(authorInfo,SWT.FILL);
	    nameLabel.setText(AnnotationPlugin.getResourceString("AE_GL_NAME"));
	    nameLabel.setBackground(colWhite);
		nameTextField=new Text(authorInfo,SWT.SINGLE|SWT.BORDER|SWT.FILL);
	    Label emailLabel=new Label(authorInfo,SWT.FILL);
	    emailLabel.setText(AnnotationPlugin.getResourceString("AE_GL_EMAIL"));
	    emailLabel.setBackground(colWhite);
		eMailTextField=new Text(authorInfo,SWT.SINGLE|SWT.BORDER|SWT.FILL);
	    Label pointsLabel=new Label(authorInfo,SWT.FILL);
	    pointsLabel.setText(AnnotationPlugin.getResourceString("AE_GL_POINTS"));
	    pointsLabel.setBackground(colWhite);
		maxPointsTextField=new Text(authorInfo,SWT.SINGLE|SWT.BORDER|SWT.FILL);

	    //********Annotation Information Group********
		FillLayout annoInfLayout=new FillLayout(SWT.VERTICAL);
	    annoInfLayout.marginHeight=10;
	    annoInfLayout.marginWidth=10;
	    annoInfLayout.spacing=3;
			
	    Group annoInfo=new Group(composite,SWT.FILL);
	    annoInfo.setText(AnnotationPlugin.getResourceString("AE_GL_GRINFO"));
	    annoInfo.setLayout(annoInfLayout);
	    annoInfo.setLayoutData(new RowData(350,85));
	    annoInfo.setBackground(colWhite);

	    annoCountLabel=new Label(annoInfo,SWT.FILL);
	    annoCountLabel.setBackground(colWhite);
	    totalModification=new Label(annoInfo,SWT.FILL);
	    totalModification.setBackground(colWhite);
	    totalScore=new Label(annoInfo,SWT.FILL);
	    totalScore.setBackground(colWhite);

	    //*************Save Button***************
	    Button saveModifications=new Button(composite,SWT.PUSH);
	    saveModifications.setText(AnnotationPlugin.getResourceString("AE_GL_BUTMOD"));
	    saveModifications.addSelectionListener(new SelectionListener(){
			public void widgetSelected(SelectionEvent e) {
				widgetDefaultSelected(e);
			}

			public void widgetDefaultSelected(SelectionEvent e) {
				AnnotationEditor.this.saveModification();		
			}});
	        
	    //***********Editor Page Options********
		globEditorPageIndex = addPage(composite);
		setPageText(globEditorPageIndex, 
				AnnotationPlugin.getResourceString("AR_ET_GLOBTAB"));
		refreshGlobalEditor();
	}
	
	/**
	 * Saves modifactions from "Global" tab of the Editor.
	 */
	protected void saveModification() {
		
		String name=null;
		String email=null;
		String points=null;
		
		name=nameTextField.getText();
		email=eMailTextField.getText();
		points=maxPointsTextField.getText();
		
		//Check if the maxPoints value is parsable
		try{
			Integer.parseInt(maxPointsTextField.getText());
		}catch(Exception e){
			resourceMessage("AE_E_INVPOINTS");
			return;
		}
		
		if(annoFile!=null){
			//Set heather item 'Name'
			annoFile.addHeaderItem(AnnotationPlugin.getDefault().
					getPreferenceStore().getString(AnnoPreferencePage.D_NAME_KEY),name);
			//'eMail'
			annoFile.addHeaderItem(AnnotationPlugin.getDefault().
					getPreferenceStore().getString(AnnoPreferencePage.D_EMAIL_KEY),email);
			//'Points'
			annoFile.addHeaderItem(AnnotationPlugin.getDefault().
					getPreferenceStore().getString(AnnoPreferencePage.D_POINTS_KEY),points);
			if(!annoFile.saveFile()){
				resourceMessage("AE_E_SAVEF");
			}
		}else{
			resourceMessage("AE_E_NOTINIT");
		}
	}

	/**
	 * Refreshes the Information shown in the global Editor.
	 */
	private void refreshGlobalEditor() {
		Hashtable heather=annoFile.getHeaderItems();
		List items=annoFile.getAnnotationItems();
		
		//Reading heather value 'Name'
		String name=(String)heather.get(AnnotationPlugin.getDefault().
				getPreferenceStore().getString(AnnoPreferencePage.D_NAME_KEY));
		//'eMail'
		String email=(String)heather.get(AnnotationPlugin.getDefault().
				getPreferenceStore().getString(AnnoPreferencePage.D_EMAIL_KEY));
		//'Points'
		String points=(String)heather.get(AnnotationPlugin.getDefault().
				getPreferenceStore().getString(AnnoPreferencePage.D_POINTS_KEY));
		
		nameTextField.setText(name);
		eMailTextField.setText(email);
		maxPointsTextField.setText(points);
		
		annoCountLabel.setText(
				AnnotationPlugin.getResourceString("AE_GL_ANNOS")+" "+items.size());
		
		/*Calculate total score modification & total Score*/
		try{
			Iterator i=items.iterator();
		
			int mod=0;
			while(i.hasNext()){
				AnnoObject o=(AnnoObject)i.next();
				mod+=o.getPoints();
			}
			totalModification.setText(
					AnnotationPlugin.getResourceString("AE_GL_MODIF")+" "+mod);
			
			/*Calculate total score*/
			mod+=Integer.parseInt(points);
			totalScore.setText(
					AnnotationPlugin.getResourceString("AE_GL_SCORE")+" "+mod);			
					
		}catch(Exception e){
			resourceMessage("AE_E_CALCPM");
		}	
	}
	
	/**
	 * Loads a message from resource bundle using key and
	 * shows it via showMessage.
	 * @param key
	 */
	private void resourceMessage(String key){
		MessageDialog.openInformation(
				viewer.getControl().getShell(),
				AnnotationPlugin.getResourceString("AE_MESSAGE_TOP"),
				AnnotationPlugin.getResourceString(key));
	}

	/**
	 * Loads a message from resource bundle using key and
	 * shows it via showMessage.
	 * @param key
	 * @param add
	 */
	private void resourceMessage(String key, String add){
		MessageDialog.openInformation(
				viewer.getControl().getShell(),
				AnnotationPlugin.getResourceString("AE_MESSAGE_TOP"),
				AnnotationPlugin.getResourceString(key)+add);
	}

	/**
	 * Not used.
	 */
	public void doSave(IProgressMonitor monitor) {}
	
	/**
	 * Not used. 
	 */
	public void doSaveAs() {}

	/**
	 * Not used in this implementation.
	 * @param marker
	 */
	public void gotoMarker(IMarker marker) {}
	
	/**
	 * Initiaize the editor
	 */
	public void init(IEditorSite site, IEditorInput editorInput) throws PartInitException {
		
		if (!(editorInput instanceof IFileEditorInput))
			throw new PartInitException(AnnotationPlugin.
					getResourceString("AE_PI_PARTINIT"));
		super.init(site, editorInput);
		
		file=((IFileEditorInput)editorInput).getFile();
		this.editorInput=editorInput;
		
		/*Read Inputfile*/
		if(!annoFile.readFile(file.getFullPath().toString())){
				resourceMessage("AE_E_READF",file.getFullPath().toString());
		}else {
			fileOK=true;
			AnnotationViewUtils.openAnnotationView(new AnnotationFile(file.getFullPath().toString()));
			contentProvider.setAnnotationList(annoFile.getAnnotationItems());
		}
		makeActions();	
	}
	
	/**
	 * Saving our editor is useless- save is no allowed
	 */
	public boolean isSaveAsAllowed() {
		return false;
	}
	
	/**
	 * Handles page changes
	 */
	protected void pageChange(int newPageIndex) {
		super.pageChange(newPageIndex);
	}
	
	/**
	 * This Method is called if any resource is changed. 
	 * Nevertheless it only handles the close event of a 
	 * Project, and changes in the Sourcefile (.ann)
	 */
	public void resourceChanged(final IResourceChangeEvent event){
		
		//Close editor on project close 
		if(event.getType() == IResourceChangeEvent.PRE_CLOSE){
			Display.getDefault().asyncExec(new Runnable(){
				public void run(){
					IWorkbenchPage[] pages = getSite().getWorkbenchWindow().getPages();
					for (int i = 0; i<pages.length; i++){
						if(file.getProject().equals(event.getResource())){
							IEditorPart editorPart = pages[i].findEditor(editorInput);
							pages[i].closeEditor(editorPart,true);
						}
					}
				}            
			});
		}
		
		//changes in the sourcefile
		if(event.getType()==IResourceChangeEvent.POST_CHANGE){
			//check if it was our resource that was changed 
			IResource res=null;
			IResourceDelta delta=event.getDelta().findMember(file.getFullPath());
			if(delta!=null) res=delta.getResource();
			
			if(res!=null){
				if(annoFile.refresh()) contentProvider.setAnnotationList(annoFile.getAnnotationItems());
				viewer.refresh();
				refreshGlobalEditor();
			}	
		}
	}
	
	
	/**
	 * Remove the listenerbindings and dispose the color. 
	 */
	public void dispose() {
		ResourcesPlugin.getWorkspace().removeResourceChangeListener(this);
		colWhite.dispose();
		super.dispose();
	}
	
	/**
	 * Hook the double click action into the listener system of the viewer.
	 */
	private void hookDoubleClickAction() {
		viewer.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {
				doubleClickAction.run();
			}
		});
	}
		
	/**
	 * Creates actions used in the editor tab
	 * + Doubleclick: Edit marked element.
	 * + Remove: Removes marked element
	 */
	private void makeActions() {
		//********* Remove Action***********
		actionRemove = new Action() {
			public void run() {
				ISelection selection = viewer.getSelection();
				AnnoObject obj = (AnnoObject)((IStructuredSelection)selection).getFirstElement();
				if(annoFile==null){
					resourceMessage("AE_W_LISTUNINIT");
					return;
				}
				if(obj==null) return;
				if(!annoFile.removeAnnotationItem(obj)) resourceMessage("AE_W_NOTREMOVE");
				viewer.refresh();
				if(!annoFile.saveFile()){
					resourceMessage("AE_E_SAVEF");
				} 
			}
		};
		
		actionRemove.setText(AnnotationPlugin.getResourceString("AE_EL_REMOVE"));
		actionRemove.setToolTipText(AnnotationPlugin.getResourceString("AE_EL_REMOVETT"));
		actionRemove.setImageDescriptor(AnnotationPlugin.getDefault().getImageDescriptor(AnnotationPlugin.IMG_DEL));
		
		//**********DoubleClick Action************
		doubleClickAction = new Action() {
			public void run() {
				//Open the selected item in an edit dialog
				Shell s=new Shell();
				ISelection selection = viewer.getSelection();
				AnnoObject obj = (AnnoObject)((IStructuredSelection)selection).getFirstElement();
				if(obj==null) return;
				(new AnnoInputDialog(s,obj,annoFile)).open();
			}
		};
		doubleClickAction.setText(AnnotationPlugin.getResourceString("AE_EL_EDIT"));
		doubleClickAction.setImageDescriptor(AnnotationPlugin.getDefault().getImageDescriptor(AnnotationPlugin.IMG_EDIT));	
	}
	
	/**
	 * Hook a Context Menu into the viewer's system
	 */
	private void hookContextMenu() {
		MenuManager menuMgr = new MenuManager(CMenuType,CMenuID);
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager manager) {
				fillContextMenu(manager);
			}
		});
		Menu menu = menuMgr.createContextMenu(viewer.getControl());
		viewer.getControl().setMenu(menu);
		getSite().registerContextMenu(menuMgr, viewer);
		getSite().registerContextMenu(menuMgr, null);	
	}

	/**
	 * Add the items to the context menu
	 * @param manager
	 */
	private void fillContextMenu(IMenuManager manager) {
		manager.add(doubleClickAction);
		manager.add(actionRemove);
		// Other plug-ins can contribute actions here
		manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
	}
}